# frozen_string_literal: true

require "rails"

module Dhall
	module Rails
		class Configuration < ::Rails::Application::Configuration
			attr_reader :credentials

			def initialize(*)
				super
				@credentials              = ActiveSupport::OrderedOptions.new
				@credentials.content_path = default_credentials_content_path
				@credentials.key_path     = default_credentials_key_path
			end

			def paths
				@paths ||= begin
					paths = super
					paths.add "config/database", with: "config/database.dhall"
					paths
				end
			end

			def load_database_yaml
				if (path = paths["config/database"].existent.first)
					Dhall::Rails.load_decoded("#{path} ? {}")
				else
					{}
				end
			end

			def pathname(key)
				path = paths[key].existent.first
				path.nil? ? nil : Pathname.new(path)
			end

			def database_configuration
				path = pathname("config/database")

				return Dhall::Rails.load_decoded("#{path} ? {}") if path&.exist?

				# Value from ENV['DATABASE_URL'] is set to default database connection
				# by Active Record.
				return {} if ENV["DATABASE_URL"]

				raise "Could not load database configuration. No such file - " \
						"#{paths["config/database"].instance_variable_get(:@paths)}"
			rescue => e
				raise e, "Cannot load database configuration:\n#{e.message}", e.backtrace
			end

			def active_storage
				as = super
				dhall_config = Pathname.new(root.join("config/storage.dhall"))
				yaml_config = Pathname.new(root.join("config/storage.yml"))

				unless dhall_config.exist? || yaml_config.exist?
					raise "Couldn't find Active Storage configuration in #{dhall_config}"
				end

				if as.service_configurations.nil? && dhall_config.exist?
					as.service_configurations = Dhall::Rails.load_decoded(dhall_config)
				end

				as
			end

			def default_credentials_content_path
				if credentials_available_for_current_env?
					root&.join("config", "credentials", "#{::Rails.env}.dhall.enc")
				else
					root&.join("config", "credentials.dhall.enc")
				end
			end

			def default_credentials_key_path
				if credentials_available_for_current_env?
					root&.join("config", "credentials", "#{::Rails.env}.key")
				else
					root&.join("config", "master.key")
				end
			end

			def credentials_available_for_current_env?
				File.exist?(
					root&.join("config", "credentials", "#{::Rails.env}.dhall.enc").to_s
				)
			end
		end
	end
end
